import time
import sys
import os
from typing import Match
sys.path.append(os.path.join(os.path.dirname(__file__),'../'))
from conclave.task_runner import TaskRunner
from conclave.step import Step
from conclave.world import World
import multiprocessing
from conclave.monitor import Monitor
from conclave.task_runner import TaskRunner
from conclave.task_logger import TaskLogger

@Step(pattern="^match group (.+) and (.+)$")
def matchGroup(logger: TaskLogger, world: World,match: Match[str]):
    logger.log(f"matchgroup called")
    group1 = match.group(1)
    group2 = match.group(2)
    logger.log(f"match group 1: {group1}")
    logger.log(f"match group 2: {group2}")

if __name__ == '__main__':
    print(f"cpu count: {multiprocessing.cpu_count()}")
    tr = TaskRunner(debugMode=True)
    testResult = tr.run(["match_group.feature"])

    print("\nprogram elapsed time :", testResult.elapsed)

    if not testResult.success:
        print(f"Test failed")
        os._exit(1)


from .feedback_schema import ScenarioFeedback, StepFeedback

class FeedbackAdapter:

    def __init__(self):
        raise NotImplementedError

    def onNotifyScenario(self,schema: ScenarioFeedback):
        raise NotImplementedError
    
    def onNotifyStep(self,schema: StepFeedback):
        raise NotImplementedError


class NullFeedbackAdapter(FeedbackAdapter):

    def __init__(self):
        pass

    def onNotifyScenario(self,schema: ScenarioFeedback):
        print(f"scenario: {schema}")
    
    def onNotifyStep(self,schema: StepFeedback):
        print(f"step: {schema}")


from dataclasses import dataclass
from typing import Any

@dataclass
class StepResult:
    elapsed: float
    result: Any
    error: str
    threadId: int
    pid: int
    start: Any
    end: Any
    log: str


from dataclasses import asdict
from multiprocessing import Process
import multiprocessing
import os
import time
import queue
from typing import Any

from .feedback_schema import BaseFeedback, ScenarioFeedback, StepFeedback
from .feedback_adapter import FeedbackAdapter

class Feedback:

    def __init__(self) -> None:
        self.messageQueue = multiprocessing.Manager().Queue()
        self.adapters: list[FeedbackAdapter] = []
        self.process = Process(daemon=True,target=self.runFeedback, args=(self.messageQueue,))

    def runFeedback(self,q: multiprocessing.Queue):
        selfPid = os.getpid()
        print(f"feedback process running: {selfPid}")
        while True:
            try:
                msg = q.get(block=True)
                #print(f"message recv: {msg}")
                if msg == "STOP":
                    break
                else:
                    for adapter in self.adapters:
                        fb = asdict(BaseFeedback())
                        fb = BaseFeedback(**{k:(msg[k] if k in msg else v) for k,v in fb.items()})
                        if fb.type == "ScenarioFeedback":
                            sfb = asdict(ScenarioFeedback())
                            sfb = ScenarioFeedback(**{k:(msg[k] if k in msg else v) for k,v in sfb.items()})
                            try:
                                adapter.onNotifyScenario(sfb)
                            except Exception as e:
                                print(f"notify feedback failed: {e}")
                        elif fb.type == "StepFeedback":
                            sfb = asdict(StepFeedback())
                            sfb = StepFeedback(**{k:(msg[k] if k in msg else v) for k,v in sfb.items()})
                            try:
                                adapter.onNotifyStep(sfb)
                            except Exception as e:
                                print(f"notify feedback failed: {e}")
            except queue.Empty:
                pass

    def startFeedback(self):
        self.process.start()
    
    def stopFeedback(self):
        self.messageQueue.put_nowait("STOP")
        self.process.join(timeout=120)
    
    def notify(self, msg: Any):
        try:
            self.messageQueue.put_nowait(msg)
        except Exception as e:
            pass
    
    def addAdapter(self,adapter: FeedbackAdapter):
        self.adapters.append(adapter)
    
